"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Db = void 0;
const apollo_boost_1 = require("apollo-boost");
const apollo_link_http_1 = require("apollo-link-http");
const cross_fetch_1 = __importDefault(require("cross-fetch"));
const utc_1 = __importDefault(require("dayjs/plugin/utc"));
const dayjs_1 = __importDefault(require("dayjs"));
const child_process_1 = require("child_process");
const dynamic_1 = require("set-interval-async/dynamic");
const path_1 = __importDefault(require("path"));
const p_retry_1 = __importDefault(require("p-retry"));
const util_1 = __importDefault(require("@cardano-graphql/util"));
dayjs_1.default.extend(utc_1.default);
class Db {
    constructor(hasuraUri) {
        this.hasuraUri = hasuraUri;
        this.hasuraClient = new apollo_boost_1.ApolloClient({
            cache: new apollo_boost_1.InMemoryCache({
                addTypename: false
            }),
            defaultOptions: {
                query: {
                    fetchPolicy: 'network-only'
                }
            },
            link: apollo_link_http_1.createHttpLink({
                uri: `${this.hasuraUri}/v1/graphql`,
                fetch: cross_fetch_1.default,
                headers: {
                    'X-Hasura-Role': 'cardano-graphql'
                }
            })
        });
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.applySchemaAndMetadata();
            this.monitorDbState();
        });
    }
    shutdown() {
        return __awaiter(this, void 0, void 0, function* () {
            yield dynamic_1.clearIntervalAsync(this.monitorTimer);
        });
    }
    monitorDbState() {
        this.monitorTimer = dynamic_1.setIntervalAsync(() => __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.getMeta();
            }
            catch (error) {
                if (error.message === 'GraphQL error: field "cardano" not found in type: \'query_root\'' || error.message === 'GraphQL error: database query error') {
                    console.warn('Re-applying PostgreSQL migrations and Hasura metadata');
                    yield this.applySchemaAndMetadata();
                }
                else {
                    console.error(error);
                }
            }
        }), 10000);
    }
    getMeta() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.hasuraClient.query({
                query: apollo_boost_1.gql `query {
          cardano {
              tip {
                  forgedAt
              }
          }}`
            });
            const { tip } = (_a = result.data) === null || _a === void 0 ? void 0 : _a.cardano[0];
            const currentUtc = dayjs_1.default().utc();
            const tipUtc = dayjs_1.default.utc(tip.forgedAt);
            return {
                initialized: tipUtc.isAfter(currentUtc.subtract(120, 'second')),
                syncPercentage: (tipUtc.valueOf() / currentUtc.valueOf()) * 100
            };
        });
    }
    applySchemaAndMetadata() {
        return __awaiter(this, void 0, void 0, function* () {
            yield p_retry_1.default(() => __awaiter(this, void 0, void 0, function* () {
                yield this.hasuraCli('migrate apply --down all');
                yield this.hasuraCli('migrate apply --up all');
                yield this.hasuraCli('metadata clear');
                yield this.hasuraCli('metadata apply');
            }), {
                factor: 1.75,
                retries: 9,
                onFailedAttempt: util_1.default.onFailedAttemptFor('Applying PostgreSQL schema and Hasura metadata')
            });
        });
    }
    hasuraCli(command) {
        return new Promise((resolve, reject) => {
            child_process_1.exec(`hasura --skip-update-check --project ${path_1.default.resolve(__dirname, '..', 'hasura', 'project')} --endpoint ${this.hasuraUri} ${command}`, (error, stdout) => {
                if (error) {
                    reject(error);
                }
                console.log(stdout);
                resolve();
            });
        });
    }
}
exports.Db = Db;
//# sourceMappingURL=Db.js.map